# python build stubs for package dither_go
# File is generated by gopy. Do not edit.
# gopy build -no-make -output=dither_go/bindings -vm=python3 ./dither_go github.com/tfuxu/dither-gopy

from pybindgen import retval, param, Function, Module
import sys

class CheckedFunction(Function):
    def __init__(self, *a, **kw):
        super(CheckedFunction, self).__init__(*a, **kw)
        self._failure_expression = kw.get('failure_expression', '')
        self._failure_cleanup = kw.get('failure_cleanup', '')

    def set_failure_expression(self, expr):
        self._failure_expression = expr

    def set_failure_cleanup(self, expr):
        self._failure_cleanup = expr

    def generate_call(self):
        super(CheckedFunction, self).generate_call()
        check = "PyErr_Occurred()"
        if self._failure_expression:
            check = "{} && {}".format(self._failure_expression, check)
        failure_cleanup = self._failure_cleanup or None
        self.before_call.write_error_check(check, failure_cleanup)

def add_checked_function(mod, name, retval, params, failure_expression='', *a, **kw):
    fn = CheckedFunction(name, retval, params, *a, **kw)
    fn.set_failure_expression(failure_expression)
    mod._add_function_obj(fn)
    return fn

def add_checked_string_function(mod, name, retval, params, failure_expression='', *a, **kw):
    fn = CheckedFunction(name, retval, params, *a, **kw)
    fn.set_failure_cleanup('if (retval != NULL) free(retval);')
    fn.after_call.add_cleanup_code('free(retval);')
    fn.set_failure_expression(failure_expression)
    mod._add_function_obj(fn)
    return fn

mod = Module('_dither_go')
mod.add_include('"dither_go_go.h"')
mod.add_function('GoPyInit', None, [])
mod.add_function('DecRef', None, [param('int64_t', 'handle')])
mod.add_function('IncRef', None, [param('int64_t', 'handle')])
mod.add_function('NumHandles', retval('int'), [])
mod.add_function('color_Palette_CTor', retval('int64_t'), [])
mod.add_function('color_Palette_len', retval('int'), [param('int64_t', 'handle')])
mod.add_function('color_Palette_elem', retval('int64_t'), [param('int64_t', 'handle'), param('int', 'idx')])
mod.add_function('color_Palette_subslice', retval('int64_t'), [param('int64_t', 'handle'), param('int', 'st'), param('int', 'ed')])
mod.add_function('color_Palette_set', None, [param('int64_t', 'handle'), param('int', 'idx'), param('int64_t', 'value')])
mod.add_function('color_Palette_append', None, [param('int64_t', 'handle'), param('int64_t', 'value')])
mod.add_function('Slice_bool_CTor', retval('int64_t'), [])
mod.add_function('Slice_bool_len', retval('int'), [param('int64_t', 'handle')])
mod.add_function('Slice_bool_elem', retval('bool'), [param('int64_t', 'handle'), param('int', 'idx')])
mod.add_function('Slice_bool_subslice', retval('int64_t'), [param('int64_t', 'handle'), param('int', 'st'), param('int', 'ed')])
mod.add_function('Slice_bool_set', None, [param('int64_t', 'handle'), param('int', 'idx'), param('bool', 'value')])
mod.add_function('Slice_bool_append', None, [param('int64_t', 'handle'), param('bool', 'value')])
mod.add_function('Slice_byte_CTor', retval('int64_t'), [])
mod.add_function('Slice_byte_len', retval('int'), [param('int64_t', 'handle')])
mod.add_function('Slice_byte_elem', retval('uint8_t'), [param('int64_t', 'handle'), param('int', 'idx')])
mod.add_function('Slice_byte_subslice', retval('int64_t'), [param('int64_t', 'handle'), param('int', 'st'), param('int', 'ed')])
mod.add_function('Slice_byte_set', None, [param('int64_t', 'handle'), param('int', 'idx'), param('uint8_t', 'value')])
mod.add_function('Slice_byte_append', None, [param('int64_t', 'handle'), param('uint8_t', 'value')])
mod.add_function('Slice_error_CTor', retval('int64_t'), [])
mod.add_function('Slice_error_len', retval('int'), [param('int64_t', 'handle')])
mod.add_function('Slice_error_elem', retval('char*'), [param('int64_t', 'handle'), param('int', 'idx')])
mod.add_function('Slice_error_subslice', retval('int64_t'), [param('int64_t', 'handle'), param('int', 'st'), param('int', 'ed')])
mod.add_function('Slice_error_set', None, [param('int64_t', 'handle'), param('int', 'idx'), param('char*', 'value')])
mod.add_function('Slice_error_append', None, [param('int64_t', 'handle'), param('char*', 'value')])
mod.add_function('Slice_float32_CTor', retval('int64_t'), [])
mod.add_function('Slice_float32_len', retval('int'), [param('int64_t', 'handle')])
mod.add_function('Slice_float32_elem', retval('float'), [param('int64_t', 'handle'), param('int', 'idx')])
mod.add_function('Slice_float32_subslice', retval('int64_t'), [param('int64_t', 'handle'), param('int', 'st'), param('int', 'ed')])
mod.add_function('Slice_float32_set', None, [param('int64_t', 'handle'), param('int', 'idx'), param('float', 'value')])
mod.add_function('Slice_float32_append', None, [param('int64_t', 'handle'), param('float', 'value')])
mod.add_function('Slice_float64_CTor', retval('int64_t'), [])
mod.add_function('Slice_float64_len', retval('int'), [param('int64_t', 'handle')])
mod.add_function('Slice_float64_elem', retval('double'), [param('int64_t', 'handle'), param('int', 'idx')])
mod.add_function('Slice_float64_subslice', retval('int64_t'), [param('int64_t', 'handle'), param('int', 'st'), param('int', 'ed')])
mod.add_function('Slice_float64_set', None, [param('int64_t', 'handle'), param('int', 'idx'), param('double', 'value')])
mod.add_function('Slice_float64_append', None, [param('int64_t', 'handle'), param('double', 'value')])
mod.add_function('Slice_int_CTor', retval('int64_t'), [])
mod.add_function('Slice_int_len', retval('int'), [param('int64_t', 'handle')])
mod.add_function('Slice_int_elem', retval('int64_t'), [param('int64_t', 'handle'), param('int', 'idx')])
mod.add_function('Slice_int_subslice', retval('int64_t'), [param('int64_t', 'handle'), param('int', 'st'), param('int', 'ed')])
mod.add_function('Slice_int_set', None, [param('int64_t', 'handle'), param('int', 'idx'), param('int64_t', 'value')])
mod.add_function('Slice_int_append', None, [param('int64_t', 'handle'), param('int64_t', 'value')])
mod.add_function('Slice_int16_CTor', retval('int64_t'), [])
mod.add_function('Slice_int16_len', retval('int'), [param('int64_t', 'handle')])
mod.add_function('Slice_int16_elem', retval('int16_t'), [param('int64_t', 'handle'), param('int', 'idx')])
mod.add_function('Slice_int16_subslice', retval('int64_t'), [param('int64_t', 'handle'), param('int', 'st'), param('int', 'ed')])
mod.add_function('Slice_int16_set', None, [param('int64_t', 'handle'), param('int', 'idx'), param('int16_t', 'value')])
mod.add_function('Slice_int16_append', None, [param('int64_t', 'handle'), param('int16_t', 'value')])
mod.add_function('Slice_int32_CTor', retval('int64_t'), [])
mod.add_function('Slice_int32_len', retval('int'), [param('int64_t', 'handle')])
mod.add_function('Slice_int32_elem', retval('int32_t'), [param('int64_t', 'handle'), param('int', 'idx')])
mod.add_function('Slice_int32_subslice', retval('int64_t'), [param('int64_t', 'handle'), param('int', 'st'), param('int', 'ed')])
mod.add_function('Slice_int32_set', None, [param('int64_t', 'handle'), param('int', 'idx'), param('int32_t', 'value')])
mod.add_function('Slice_int32_append', None, [param('int64_t', 'handle'), param('int32_t', 'value')])
mod.add_function('Slice_int64_CTor', retval('int64_t'), [])
mod.add_function('Slice_int64_len', retval('int'), [param('int64_t', 'handle')])
mod.add_function('Slice_int64_elem', retval('int64_t'), [param('int64_t', 'handle'), param('int', 'idx')])
mod.add_function('Slice_int64_subslice', retval('int64_t'), [param('int64_t', 'handle'), param('int', 'st'), param('int', 'ed')])
mod.add_function('Slice_int64_set', None, [param('int64_t', 'handle'), param('int', 'idx'), param('int64_t', 'value')])
mod.add_function('Slice_int64_append', None, [param('int64_t', 'handle'), param('int64_t', 'value')])
mod.add_function('Slice_int8_CTor', retval('int64_t'), [])
mod.add_function('Slice_int8_len', retval('int'), [param('int64_t', 'handle')])
mod.add_function('Slice_int8_elem', retval('int8_t'), [param('int64_t', 'handle'), param('int', 'idx')])
mod.add_function('Slice_int8_subslice', retval('int64_t'), [param('int64_t', 'handle'), param('int', 'st'), param('int', 'ed')])
mod.add_function('Slice_int8_set', None, [param('int64_t', 'handle'), param('int', 'idx'), param('int8_t', 'value')])
mod.add_function('Slice_int8_append', None, [param('int64_t', 'handle'), param('int8_t', 'value')])
mod.add_function('Slice_rune_CTor', retval('int64_t'), [])
mod.add_function('Slice_rune_len', retval('int'), [param('int64_t', 'handle')])
mod.add_function('Slice_rune_elem', retval('int32_t'), [param('int64_t', 'handle'), param('int', 'idx')])
mod.add_function('Slice_rune_subslice', retval('int64_t'), [param('int64_t', 'handle'), param('int', 'st'), param('int', 'ed')])
mod.add_function('Slice_rune_set', None, [param('int64_t', 'handle'), param('int', 'idx'), param('int32_t', 'value')])
mod.add_function('Slice_rune_append', None, [param('int64_t', 'handle'), param('int32_t', 'value')])
mod.add_function('Slice_string_CTor', retval('int64_t'), [])
mod.add_function('Slice_string_len', retval('int'), [param('int64_t', 'handle')])
mod.add_function('Slice_string_elem', retval('char*'), [param('int64_t', 'handle'), param('int', 'idx')])
mod.add_function('Slice_string_subslice', retval('int64_t'), [param('int64_t', 'handle'), param('int', 'st'), param('int', 'ed')])
mod.add_function('Slice_string_set', None, [param('int64_t', 'handle'), param('int', 'idx'), param('char*', 'value')])
mod.add_function('Slice_string_append', None, [param('int64_t', 'handle'), param('char*', 'value')])
mod.add_function('Slice_uint_CTor', retval('int64_t'), [])
mod.add_function('Slice_uint_len', retval('int'), [param('int64_t', 'handle')])
mod.add_function('Slice_uint_elem', retval('uint64_t'), [param('int64_t', 'handle'), param('int', 'idx')])
mod.add_function('Slice_uint_subslice', retval('int64_t'), [param('int64_t', 'handle'), param('int', 'st'), param('int', 'ed')])
mod.add_function('Slice_uint_set', None, [param('int64_t', 'handle'), param('int', 'idx'), param('uint64_t', 'value')])
mod.add_function('Slice_uint_append', None, [param('int64_t', 'handle'), param('uint64_t', 'value')])
mod.add_function('Slice_uint16_CTor', retval('int64_t'), [])
mod.add_function('Slice_uint16_len', retval('int'), [param('int64_t', 'handle')])
mod.add_function('Slice_uint16_elem', retval('uint16_t'), [param('int64_t', 'handle'), param('int', 'idx')])
mod.add_function('Slice_uint16_subslice', retval('int64_t'), [param('int64_t', 'handle'), param('int', 'st'), param('int', 'ed')])
mod.add_function('Slice_uint16_set', None, [param('int64_t', 'handle'), param('int', 'idx'), param('uint16_t', 'value')])
mod.add_function('Slice_uint16_append', None, [param('int64_t', 'handle'), param('uint16_t', 'value')])
mod.add_function('Slice_uint32_CTor', retval('int64_t'), [])
mod.add_function('Slice_uint32_len', retval('int'), [param('int64_t', 'handle')])
mod.add_function('Slice_uint32_elem', retval('uint32_t'), [param('int64_t', 'handle'), param('int', 'idx')])
mod.add_function('Slice_uint32_subslice', retval('int64_t'), [param('int64_t', 'handle'), param('int', 'st'), param('int', 'ed')])
mod.add_function('Slice_uint32_set', None, [param('int64_t', 'handle'), param('int', 'idx'), param('uint32_t', 'value')])
mod.add_function('Slice_uint32_append', None, [param('int64_t', 'handle'), param('uint32_t', 'value')])
mod.add_function('Slice_uint64_CTor', retval('int64_t'), [])
mod.add_function('Slice_uint64_len', retval('int'), [param('int64_t', 'handle')])
mod.add_function('Slice_uint64_elem', retval('uint64_t'), [param('int64_t', 'handle'), param('int', 'idx')])
mod.add_function('Slice_uint64_subslice', retval('int64_t'), [param('int64_t', 'handle'), param('int', 'st'), param('int', 'ed')])
mod.add_function('Slice_uint64_set', None, [param('int64_t', 'handle'), param('int', 'idx'), param('uint64_t', 'value')])
mod.add_function('Slice_uint64_append', None, [param('int64_t', 'handle'), param('uint64_t', 'value')])
mod.add_function('Slice_uint8_CTor', retval('int64_t'), [])
mod.add_function('Slice_uint8_len', retval('int'), [param('int64_t', 'handle')])
mod.add_function('Slice_uint8_elem', retval('uint8_t'), [param('int64_t', 'handle'), param('int', 'idx')])
mod.add_function('Slice_uint8_subslice', retval('int64_t'), [param('int64_t', 'handle'), param('int', 'st'), param('int', 'ed')])
mod.add_function('Slice_uint8_set', None, [param('int64_t', 'handle'), param('int', 'idx'), param('uint8_t', 'value')])
mod.add_function('Slice_uint8_append', None, [param('int64_t', 'handle'), param('uint8_t', 'value')])
mod.add_function('Slice_color_Color_CTor', retval('int64_t'), [])
mod.add_function('Slice_color_Color_len', retval('int'), [param('int64_t', 'handle')])
mod.add_function('Slice_color_Color_elem', retval('int64_t'), [param('int64_t', 'handle'), param('int', 'idx')])
mod.add_function('Slice_color_Color_subslice', retval('int64_t'), [param('int64_t', 'handle'), param('int', 'st'), param('int', 'ed')])
mod.add_function('Slice_color_Color_set', None, [param('int64_t', 'handle'), param('int', 'idx'), param('int64_t', 'value')])
mod.add_function('Slice_color_Color_append', None, [param('int64_t', 'handle'), param('int64_t', 'value')])
add_checked_function(mod, 'dither_go_CreatePalette', retval('int64_t'), [param('int64_t', 'colors')])
add_checked_function(mod, 'dither_go_CreateRGBA', retval('int64_t'), [param('uint8_t', 'r'), param('uint8_t', 'g'), param('uint8_t', 'b'), param('uint8_t', 'a')])
add_checked_function(mod, 'dither_go_OpenImage', retval('int64_t'), [param('char*', 'path')])
add_checked_function(mod, 'dither_go_SaveImage', retval('char*'), [param('int64_t', 'img_data'), param('char*', 'output_path'), param('char*', 'encode_format')])
mod.add_function('Slice_Slice_uint_CTor', retval('int64_t'), [])
mod.add_function('Slice_Slice_uint_len', retval('int'), [param('int64_t', 'handle')])
mod.add_function('Slice_Slice_uint_elem', retval('int64_t'), [param('int64_t', 'handle'), param('int', 'idx')])
mod.add_function('Slice_Slice_uint_subslice', retval('int64_t'), [param('int64_t', 'handle'), param('int', 'st'), param('int', 'ed')])
mod.add_function('Slice_Slice_uint_set', None, [param('int64_t', 'handle'), param('int', 'idx'), param('int64_t', 'value')])
mod.add_function('Slice_Slice_uint_append', None, [param('int64_t', 'handle'), param('int64_t', 'value')])
mod.add_function('dither_Atkinson', retval('int64_t'), [])
mod.add_function('dither_Set_Atkinson', None, [param('int64_t', 'val')])
mod.add_function('dither_Burkes', retval('int64_t'), [])
mod.add_function('dither_Set_Burkes', None, [param('int64_t', 'val')])
mod.add_function('dither_ClusteredDot4x4', retval('int64_t'), [])
mod.add_function('dither_Set_ClusteredDot4x4', None, [param('int64_t', 'val')])
mod.add_function('dither_ClusteredDot6x6', retval('int64_t'), [])
mod.add_function('dither_Set_ClusteredDot6x6', None, [param('int64_t', 'val')])
mod.add_function('dither_ClusteredDot6x6_2', retval('int64_t'), [])
mod.add_function('dither_Set_ClusteredDot6x6_2', None, [param('int64_t', 'val')])
mod.add_function('dither_ClusteredDot6x6_3', retval('int64_t'), [])
mod.add_function('dither_Set_ClusteredDot6x6_3', None, [param('int64_t', 'val')])
mod.add_function('dither_ClusteredDot8x8', retval('int64_t'), [])
mod.add_function('dither_Set_ClusteredDot8x8', None, [param('int64_t', 'val')])
mod.add_function('dither_ClusteredDotDiagonal16x16', retval('int64_t'), [])
mod.add_function('dither_Set_ClusteredDotDiagonal16x16', None, [param('int64_t', 'val')])
mod.add_function('dither_ClusteredDotDiagonal6x6', retval('int64_t'), [])
mod.add_function('dither_Set_ClusteredDotDiagonal6x6', None, [param('int64_t', 'val')])
mod.add_function('dither_ClusteredDotDiagonal8x8', retval('int64_t'), [])
mod.add_function('dither_Set_ClusteredDotDiagonal8x8', None, [param('int64_t', 'val')])
mod.add_function('dither_ClusteredDotDiagonal8x8_2', retval('int64_t'), [])
mod.add_function('dither_Set_ClusteredDotDiagonal8x8_2', None, [param('int64_t', 'val')])
mod.add_function('dither_ClusteredDotDiagonal8x8_3', retval('int64_t'), [])
mod.add_function('dither_Set_ClusteredDotDiagonal8x8_3', None, [param('int64_t', 'val')])
mod.add_function('dither_ClusteredDotHorizontalLine', retval('int64_t'), [])
mod.add_function('dither_Set_ClusteredDotHorizontalLine', None, [param('int64_t', 'val')])
mod.add_function('dither_ClusteredDotSpiral5x5', retval('int64_t'), [])
mod.add_function('dither_Set_ClusteredDotSpiral5x5', None, [param('int64_t', 'val')])
mod.add_function('dither_ClusteredDotVerticalLine', retval('int64_t'), [])
mod.add_function('dither_Set_ClusteredDotVerticalLine', None, [param('int64_t', 'val')])
mod.add_function('dither_FalseFloydSteinberg', retval('int64_t'), [])
mod.add_function('dither_Set_FalseFloydSteinberg', None, [param('int64_t', 'val')])
mod.add_function('dither_FloydSteinberg', retval('int64_t'), [])
mod.add_function('dither_Set_FloydSteinberg', None, [param('int64_t', 'val')])
mod.add_function('dither_Horizontal3x5', retval('int64_t'), [])
mod.add_function('dither_Set_Horizontal3x5', None, [param('int64_t', 'val')])
mod.add_function('dither_JarvisJudiceNinke', retval('int64_t'), [])
mod.add_function('dither_Set_JarvisJudiceNinke', None, [param('int64_t', 'val')])
mod.add_function('dither_Sierra', retval('int64_t'), [])
mod.add_function('dither_Set_Sierra', None, [param('int64_t', 'val')])
mod.add_function('dither_Sierra2', retval('int64_t'), [])
mod.add_function('dither_Set_Sierra2', None, [param('int64_t', 'val')])
mod.add_function('dither_Sierra2_4A', retval('int64_t'), [])
mod.add_function('dither_Set_Sierra2_4A', None, [param('int64_t', 'val')])
mod.add_function('dither_Sierra3', retval('int64_t'), [])
mod.add_function('dither_Set_Sierra3', None, [param('int64_t', 'val')])
mod.add_function('dither_SierraLite', retval('int64_t'), [])
mod.add_function('dither_Set_SierraLite', None, [param('int64_t', 'val')])
mod.add_function('dither_Simple2D', retval('int64_t'), [])
mod.add_function('dither_Set_Simple2D', None, [param('int64_t', 'val')])
mod.add_function('dither_StevenPigeon', retval('int64_t'), [])
mod.add_function('dither_Set_StevenPigeon', None, [param('int64_t', 'val')])
mod.add_function('dither_Stucki', retval('int64_t'), [])
mod.add_function('dither_Set_Stucki', None, [param('int64_t', 'val')])
mod.add_function('dither_TwoRowSierra', retval('int64_t'), [])
mod.add_function('dither_Set_TwoRowSierra', None, [param('int64_t', 'val')])
mod.add_function('dither_Vertical5x3', retval('int64_t'), [])
mod.add_function('dither_Set_Vertical5x3', None, [param('int64_t', 'val')])
mod.add_function('dither_Ditherer_CTor', retval('int64_t'), [])
mod.add_function('dither_Ditherer_Matrix_Get', retval('int64_t'), [param('int64_t', 'handle')])
mod.add_function('dither_Ditherer_Matrix_Set', None, [param('int64_t', 'handle'), param('int64_t', 'val')])
mod.add_function('dither_Ditherer_Special_Get', retval('int64_t'), [param('int64_t', 'handle')])
mod.add_function('dither_Ditherer_Special_Set', None, [param('int64_t', 'handle'), param('int64_t', 'val')])
mod.add_function('dither_Ditherer_SingleThreaded_Get', retval('bool'), [param('int64_t', 'handle')])
mod.add_function('dither_Ditherer_SingleThreaded_Set', None, [param('int64_t', 'handle'), param('bool', 'val')])
mod.add_function('dither_Ditherer_Serpentine_Get', retval('bool'), [param('int64_t', 'handle')])
mod.add_function('dither_Ditherer_Serpentine_Set', None, [param('int64_t', 'handle'), param('bool', 'val')])
add_checked_function(mod, 'dither_Ditherer_SetBayer', None, [param('int64_t', '_handle'), param('uint64_t', 'x'), param('uint64_t', 'y'), param('float', 'strength'), param('bool', 'goRun')])
add_checked_function(mod, 'dither_Ditherer_SetOrdered', None, [param('int64_t', '_handle'), param('int64_t', 'odm'), param('float', 'strength'), param('bool', 'goRun')])
add_checked_function(mod, 'dither_Ditherer_SetRandomGrayscale', None, [param('int64_t', '_handle'), param('float', 'min'), param('float', 'max'), param('bool', 'goRun')])
add_checked_function(mod, 'dither_Ditherer_SetRandomRGB', None, [param('int64_t', '_handle'), param('float', 'minR'), param('float', 'maxR'), param('float', 'minG'), param('float', 'maxG'), param('float', 'minB'), param('float', 'maxB'), param('bool', 'goRun')])
add_checked_function(mod, 'dither_Ditherer_ClearMapper', None, [param('int64_t', '_handle'), param('bool', 'goRun')])
add_checked_function(mod, 'dither_Ditherer_GetPalette', retval('int64_t'), [param('int64_t', '_handle')])
add_checked_function(mod, 'dither_Ditherer_Dither', retval('int64_t'), [param('int64_t', '_handle'), param('int64_t', 'src')])
add_checked_function(mod, 'dither_Ditherer_GetColorModel', retval('int64_t'), [param('int64_t', '_handle')])
add_checked_function(mod, 'dither_Ditherer_DitherCopy', retval('int64_t'), [param('int64_t', '_handle'), param('int64_t', 'src')])
add_checked_function(mod, 'dither_Ditherer_DitherPaletted', retval('int64_t'), [param('int64_t', '_handle'), param('int64_t', 'src')])
add_checked_function(mod, 'dither_Ditherer_Draw', None, [param('int64_t', '_handle'), param('int64_t', 'dst'), param('int64_t', 'r'), param('int64_t', 'src'), param('int64_t', 'sp'), param('bool', 'goRun')])
add_checked_function(mod, 'dither_Ditherer_Quantize', retval('int64_t'), [param('int64_t', '_handle'), param('int64_t', 'p'), param('int64_t', 'm')])
mod.add_function('dither_OrderedDitherMatrix_CTor', retval('int64_t'), [])
mod.add_function('dither_OrderedDitherMatrix_Matrix_Get', retval('int64_t'), [param('int64_t', 'handle')])
mod.add_function('dither_OrderedDitherMatrix_Matrix_Set', None, [param('int64_t', 'handle'), param('int64_t', 'val')])
mod.add_function('dither_OrderedDitherMatrix_Max_Get', retval('uint64_t'), [param('int64_t', 'handle')])
mod.add_function('dither_OrderedDitherMatrix_Max_Set', None, [param('int64_t', 'handle'), param('uint64_t', 'val')])
mod.add_function('dither_ErrorDiffusionMatrix_CTor', retval('int64_t'), [])
mod.add_function('dither_ErrorDiffusionMatrix_len', retval('int'), [param('int64_t', 'handle')])
mod.add_function('dither_ErrorDiffusionMatrix_elem', retval('int64_t'), [param('int64_t', 'handle'), param('int', 'idx')])
mod.add_function('dither_ErrorDiffusionMatrix_subslice', retval('int64_t'), [param('int64_t', 'handle'), param('int', 'st'), param('int', 'ed')])
mod.add_function('dither_ErrorDiffusionMatrix_set', None, [param('int64_t', 'handle'), param('int', 'idx'), param('int64_t', 'value')])
mod.add_function('dither_ErrorDiffusionMatrix_append', None, [param('int64_t', 'handle'), param('int64_t', 'value')])
add_checked_function(mod, 'dither_ErrorDiffusionMatrix_CurrentPixel', retval('int64_t'), [param('int64_t', '_handle')])
add_checked_function(mod, 'dither_NewDitherer', retval('int64_t'), [param('int64_t', 'palette')])
add_checked_function(mod, 'dither_ErrorDiffusionStrength', retval('int64_t'), [param('int64_t', 'edm'), param('float', 'strength')])
add_checked_function(mod, 'dither_RoundClamp', retval('uint16_t'), [param('float', 'i')])

mod.generate(open('dither_go.c', 'w'))




# Copyright 2023, tfuxu <https://github.com/tfuxu>
# SPDX-License-Identifier: GPL-3.0-or-later

from dataclasses import asdict
from typing import List, Tuple

from simple_parsing.docstring import get_attribute_docstring

from dither_go import ErrorDiffusionMatrix, OrderedDitherMatrix
from dither_go.matrices import ErrorDiffusers, OrderedDitherers


class MatrixUtils:  # pylint: disable=R0903
    """
    A class for error diffusion and ordered dithering matrices utility methods.
    """

    def __init__(self):
        pass

    def generate_matrices_list(self) -> Tuple[List[Tuple[str, ErrorDiffusionMatrix]], List[Tuple[str, OrderedDitherMatrix]]]:
        """
        Generates a list of available matrices in library with human-readable names.

        :returns: Two lists consisting of error diffusion and ordered dithering matrices with their display names.
        :rtype: Tuple[List[Tuple[str, ErrorDiffusionMatrix]], List[Tuple[str, OrderedDitherMatrix]]]
        """

        error_diffusers = asdict(ErrorDiffusers())
        ordered_ditherers = asdict(OrderedDitherers())

        error_matrices = []
        for key in error_diffusers:
            field_docstring = get_attribute_docstring(ErrorDiffusers, key).docstring_below
            display_name = field_docstring.strip().partition("\n")[0]

            error_matrices.append((display_name, error_diffusers[key]))

        ordered_matrices = []
        for key in ordered_ditherers:
            field_docstring = get_attribute_docstring(OrderedDitherers, key).docstring_below
            display_name = field_docstring.strip().partition("\n")[0]

            ordered_matrices.append((display_name, ordered_ditherers[key]))

        return error_matrices, ordered_matrices



# Copyright 2023, popatam, bjia56, tfuxu <https://github.com/tfuxu>
# SPDX-License-Identifier: GPL-3.0-or-later

import json
import os
import shutil
import subprocess
import sys

from setuptools import Extension, setup
from setuptools.command.build_ext import build_ext

if sys.platform == 'darwin':
    # PYTHON_BINARY_PATH is setting explicitly for 310 and 311, see build_wheel.yml
    # on macos PYTHON_BINARY_PATH must be python bin installed from python.org
    PYTHON_BINARY = os.getenv("PYTHON_BINARY_PATH", sys.executable)
else:
    # windown & linux
    PYTHON_BINARY = sys.executable


def _generate_path_with_gopath() -> str:
    go_path = subprocess.check_output(["go", "env", "GOPATH"]).decode("utf-8").strip()
    path_val = f'{os.getenv("PATH")}:{go_path}/bin'

    return path_val


class CustomBuildExt(build_ext):
    def build_extension(self, ext: Extension):
        bin_path = _generate_path_with_gopath()
        go_env = json.loads(subprocess.check_output(["go", "env", "-json"]).decode("utf-8").strip())

        destination = os.path.dirname(os.path.abspath(self.get_ext_fullpath(ext.name))) + "/dither_go/bindings"
        if os.path.isdir(destination):
            # clean up destination in case it has existing build artifacts
            shutil.rmtree(destination)

        env = {
            "PATH": bin_path,
            **go_env,
            "CGO_LDFLAGS_ALLOW": ".*",
        }

        # https://stackoverflow.com/a/64706392
        if sys.platform == "win32":
            env["SYSTEMROOT"] = os.environ.get("SYSTEMROOT", "")

        if sys.platform == "darwin":
            min_ver = os.environ.get("MACOSX_DEPLOYMENT_TARGET", "")
            env["MACOSX_DEPLOYMENT_TARGET"] = min_ver
            env["CGO_LDFLAGS"] = "-mmacosx-version-min=" + min_ver
            env["CGO_CFLAGS"] = "-mmacosx-version-min=" + min_ver

        subprocess.check_call(
            [
                "gopy",
                "build",
                "-no-make",
                "-dynamic-link=True",
                "-symbols=False",
                "-output",
                destination,
                "-vm",
                PYTHON_BINARY,
                *ext.sources,
            ],
            env=env,
        )


setup(
    cmdclass={
        "build_ext": CustomBuildExt,
    },
    ext_modules=[
        Extension("dither_go", sources=["./dither_go", "github.com/tfuxu/dither-gopy"])
    ],
)



# Copyright 2023, tfuxu <https://github.com/tfuxu>
# SPDX-License-Identifier: GPL-3.0-or-later

from dataclasses import dataclass

from dither_go import ErrorDiffusionMatrix, OrderedDitherMatrix
from dither_go.bindings import dither


@dataclass(frozen=True)
class ErrorDiffusers:  # pylint: disable=R0902,C0103
    """
    Error diffusion matrices
    """

    Simple2D: ErrorDiffusionMatrix = dither.Simple2D()
    """Simple 2D"""

    FloydSteinberg: ErrorDiffusionMatrix = dither.FloydSteinberg()
    """Floyd-Steinberg"""

    FalseFloydSteinberg: ErrorDiffusionMatrix = dither.FalseFloydSteinberg()
    """False Floyd-Steinberg"""

    Stucki: ErrorDiffusionMatrix = dither.Stucki()
    """Stucki"""

    Burkes: ErrorDiffusionMatrix = dither.Burkes()
    """Burkes"""

    Atkinson: ErrorDiffusionMatrix = dither.Atkinson()
    """Atkinson"""

    JarvisJudiceNinke: ErrorDiffusionMatrix = dither.JarvisJudiceNinke()
    """Jarvis, Judice & Ninke"""

    Sierra: ErrorDiffusionMatrix = dither.Sierra()
    """Sierra"""

    Sierra2: ErrorDiffusionMatrix = dither.Sierra2()
    """
    Sierra2

    Sierra2 is another name for Two-Row Sierra.
    """

    Sierra2_4A: ErrorDiffusionMatrix = dither.Sierra2_4A()
    """
    Sierra2-4A

    Sierra2_4A (usually written as Sierra2-4A) is another name for Sierra Lite.
    """

    Sierra3: ErrorDiffusionMatrix = dither.Sierra3()
    """
    Sierra3

    Sierra3 is another name for the original Sierra matrix.
    """

    SierraLite: ErrorDiffusionMatrix = dither.SierraLite()
    """Sierra Lite"""

    TwoRowSierra: ErrorDiffusionMatrix = dither.TwoRowSierra()
    """Two-Row Sierra"""

    StevenPigeon: ErrorDiffusionMatrix = dither.StevenPigeon()
    """Pigeon"""


@dataclass(frozen=True)
class OrderedDitherers:  # pylint: disable=R0902,C0103
    """
    Ordered dither matrices
    """

    ClusteredDot4x4: OrderedDitherMatrix = dither.ClusteredDot4x4()
    """
    Clustered-Dot 4x4

    Comes from http://caca.zoy.org/study/part2.html
    It is not diagonal, so the dots form a grid.
    """

    ClusteredDot6x6: OrderedDitherMatrix = dither.ClusteredDot6x6()
    """
    Clustered-Dot 6x6

    Clustered-Dot 6x6 comes from Figure 5.9 of the book "Digital Halftoning" by
    Robert Ulichney. It can represent "37 levels of gray". It is not diagonal.
    """

    ClusteredDot6x6_2: OrderedDitherMatrix = dither.ClusteredDot6x6_2()
    """
    Clustered-Dot 6x6-2

    Clustered-Dot 6x6-2 comes from https://archive.is/71e9G. On the webpage it is
    called "central white point" while Clustered-Dot 6x6 is called "clustered dots".

    It is nearly identical to Clustered-Dot 6x6.
    """

    ClusteredDot6x6_3: OrderedDitherMatrix = dither.ClusteredDot6x6_3()
    """
    Clustered-Dot 6x6-3

    Clustered-Dot 6x6-3 comes from https://archive.is/71e9G. On the webpage it is
    called "balanced centered point".

    It is nearly identical to Clustered-Dot 6x6.
    """

    ClusteredDot8x8: OrderedDitherMatrix = dither.ClusteredDot8x8()
    """
    Clustered-Dot 8x8

    Clustered-Dot 8x8 comes from Figure 1.5 of the book "Modern Digital Halftoning, Second Edition",
    by Daniel L. Lau and Gonzalo R. Arce.
    It is like Clustered-Dot Diagonal 8x8, but is not diagonal. It can represent "65 gray-levels".
    """

    ClusteredDotDiagonal6x6: OrderedDitherMatrix = dither.ClusteredDotDiagonal6x6()
    """
    Clustered-Dot Diagonal 6x6

    Clustered-Dot Diagonal 6x6 comes from Figure 5.4 of the book "Digital Halftoning" by
    Robert Ulichney.
    In the book it's called ``M = 3``. It can represent "19 levels of gray".
    Its dimensions are 6x6, but as a diagonal matrix it is 7x7. It is called
    "Diagonal" because the resulting dot pattern is at a 45 degree angle.
    """

    ClusteredDotDiagonal8x8: OrderedDitherMatrix = dither.ClusteredDotDiagonal8x8()
    """
    Clustered-Dot Diagonal 8x8

    Comes from http://caca.zoy.org/study/part2.html
    They say it "mimics the halftoning techniques used by newspapers". It is called
    "Diagonal" because the resulting dot pattern is at a 45 degree angle.
    """

    ClusteredDotDiagonal8x8_2: OrderedDitherMatrix = dither.ClusteredDotDiagonal8x8_2()
    """
    Clustered-Dot Diagonal 8x8-2

    Clustered-Dot Diagonal 8x8-2 comes from Figure 5.4 of the book "Digital Halftoning" by
    Robert Ulichney.
    In the book it's called ``M = 4``. It can represent "33 levels of gray".
    Its dimensions are 8x8, but as a diagonal matrix it is 9x9. It is called
    "Diagonal" because the resulting dot pattern is at a 45 degree angle.

    It is almost identical to Clustered-Dot Diagonal 8x8, but it's worse because it can
    represent fewer gray levels. There is not much point in using it.
    """

    ClusteredDotDiagonal8x8_3: OrderedDitherMatrix = dither.ClusteredDotDiagonal8x8_3()
    """
    Clustered-Dot Diagonal 8x8-3

    Clustered-Dot Diagonal 8x8-3 comes from https://archive.is/71e9G. On the webpage
    it is called "diagonal ordered matrix with balanced centered points".

    It is almost identical to Clustered-Dot Diagonal 8x8, but worse because it can
    represent fewer gray levels. There is not much point in using it.

    It is called "Diagonal" because the resulting dot pattern is at a 45 degree angle.
    """

    ClusteredDotDiagonal16x16: OrderedDitherMatrix = dither.ClusteredDotDiagonal16x16()
    """
    Clustered-Dot Diagonal 16x16

    Clustered-Dot Diagonal 16x16 comes from Figure 5.4 of the book "Digital Halftoning" by
    Robert Ulichney.
    In the book it's called ``M = 8``. It can represent "129 levels of gray".
    Its dimensions are 16x16, but as a diagonal matrix it is 17x17. It is called
    "Diagonal" because the resulting dot pattern is at a 45 degree angle.
    """

    Horizontal3x5: OrderedDitherMatrix = dither.Horizontal3x5()
    """
    Horizontal 3x5

    Horizontal 3x5 is a custom rotated version of Vertical 5x3.
    """

    Vertical5x3: OrderedDitherMatrix = dither.Vertical5x3()
    """
    Vertical 5x3

    Comes from http://caca.zoy.org/study/part2.html
    They say it "creates artistic vertical line artifacts".
    """

    ClusteredDotSpiral5x5: OrderedDitherMatrix = dither.ClusteredDotSpiral5x5()
    """
    Clustered-Dot Spiral 5x5

    Clustered-Dot Spiral 5x5 comes from Figure 5.13 of the book "Digital Halftoning" by
    Robert Ulichney. It can represent "26 levels of gray". Its dimensions are 5x5.

    Instead of alternating dark and light dots like the other clustered-dot
    matrices, the dark parts grow to fill the area.
    """

    ClusteredDotHorizontalLine: OrderedDitherMatrix = dither.ClusteredDotHorizontalLine()
    """
    Clustered-Dot Horizontal Line

    Clustered-Dot Horizontal Line comes from Figure 5.13 of the book "Digital Halftoning" by
    Robert Ulichney. It can represent "37 levels of gray". Its dimensions are 6x6.

    It "clusters pixels about horizontal lines".
    """

    ClusteredDotVerticalLine: OrderedDitherMatrix = dither.ClusteredDotVerticalLine()
    """
    Clustered-Dot Vertical Line

    Clustered-Dot Vertical Line is a custom rotated version of Clustered-Dot Horizontal Line.
    """
